@@ -12,34 +12,40 @@ class ForwardOutputTest < Test::Unit::TestCase
     FileUtils.rm_rf(TMP_DIR)
     FileUtils.mkdir_p(TMP_DIR)
     @d = nil
+    @target_port = unused_port
   end
 
   def teardown
     @d.instance_shutdown if @d
+    @port = nil
   end
 
   TMP_DIR = File.join(__dir__, "../tmp/out_forward#{ENV['TEST_ENV_NUMBER']}")
 
   TARGET_HOST = '127.0.0.1'
-  TARGET_PORT = unused_port
-  CONFIG = %[
-    send_timeout 51
-    heartbeat_type udp
-    <server>
-      name test
-      host #{TARGET_HOST}
-      port #{TARGET_PORT}
-    </server>
-  ]
 
-  TARGET_CONFIG = %[
-    port #{TARGET_PORT}
-    bind #{TARGET_HOST}
-  ]
+  def config
+    %[
+      send_timeout 51
+      heartbeat_type udp
+      <server>
+        name test
+        host #{TARGET_HOST}
+        port #{@target_port}
+      </server>
+    ]
+  end
 
-  def create_driver(conf=CONFIG)
+  def target_config
+    %[
+      port #{@target_port}
+      bind #{TARGET_HOST}
+    ]
+  end
+
+  def create_driver(conf=config)
     Fluent::Test::Driver::Output.new(Fluent::Plugin::ForwardOutput) {
-      attr_reader :sent_chunk_ids, :ack_handler
+      attr_reader :sent_chunk_ids, :ack_handler, :discovery_manager
 
       def initialize
         super
@@ -60,7 +66,7 @@ class ForwardOutputTest < Test::Unit::TestCase
       <server>
         name test
         host #{TARGET_HOST}
-        port #{TARGET_PORT}
+        port #{@target_port}
       </server>
     ])
     nodes = d.instance.nodes
@@ -71,7 +77,7 @@ class ForwardOutputTest < Test::Unit::TestCase
     node = nodes.first
     assert_equal "test", node.name
     assert_equal '127.0.0.1', node.host
-    assert_equal TARGET_PORT, node.port
+    assert_equal @target_port, node.port
   end
 
   test 'configure_traditional' do
@@ -80,7 +86,7 @@ class ForwardOutputTest < Test::Unit::TestCase
       <server>
         name test
         host #{TARGET_HOST}
-        port #{TARGET_PORT}
+        port #{@target_port}
       </server>
       buffer_chunk_limit 10m
 EOL
@@ -100,7 +106,7 @@ EOL
       ack_response_timeout 20
       <server>
         host #{TARGET_HOST}
-        port #{TARGET_PORT}
+        port #{@target_port}
       </server>
     ])
     assert_equal 30, d.instance.send_timeout
@@ -110,33 +116,33 @@ EOL
   end
 
   test 'configure_udp_heartbeat' do
-    @d = d = create_driver(CONFIG + "\nheartbeat_type udp")
+    @d = d = create_driver(config + "\nheartbeat_type udp")
     assert_equal :udp, d.instance.heartbeat_type
   end
 
   test 'configure_none_heartbeat' do
-    @d = d = create_driver(CONFIG + "\nheartbeat_type none")
+    @d = d = create_driver(config + "\nheartbeat_type none")
     assert_equal :none, d.instance.heartbeat_type
   end
 
   test 'configure_expire_dns_cache' do
-    @d = d = create_driver(CONFIG + "\nexpire_dns_cache 5")
+    @d = d = create_driver(config + "\nexpire_dns_cache 5")
     assert_equal 5, d.instance.expire_dns_cache
   end
 
   test 'configure_dns_round_robin udp' do
     assert_raise(Fluent::ConfigError) do
-      create_driver(CONFIG + "\nheartbeat_type udp\ndns_round_robin true")
+      create_driver(config + "\nheartbeat_type udp\ndns_round_robin true")
     end
   end
 
   test 'configure_dns_round_robin transport' do
-    @d = d = create_driver(CONFIG + "\nheartbeat_type transport\ndns_round_robin true")
+    @d = d = create_driver(config + "\nheartbeat_type transport\ndns_round_robin true")
     assert_equal true, d.instance.dns_round_robin
   end
 
   test 'configure_dns_round_robin none' do
-    @d = d = create_driver(CONFIG + "\nheartbeat_type none\ndns_round_robin true")
+    @d = d = create_driver(config + "\nheartbeat_type none\ndns_round_robin true")
     assert_equal true, d.instance.dns_round_robin
   end
 
@@ -176,7 +182,7 @@ EOL
       #{param} #{dummy_cert_path}
       <server>
         host #{TARGET_HOST}
-        port #{TARGET_PORT}
+        port #{@target_port}
       </server>
     ]
 
@@ -195,7 +201,7 @@ EOL
         tls_cert_thumbprint a909502dd82ae41433e6f83886b00d4277a32a7b
         <server>
           host #{TARGET_HOST}
-          port #{TARGET_PORT}
+          port #{@target_port}
         </server>
       ]
 
@@ -210,7 +216,7 @@ EOL
         transport tls
         <server>
           host #{TARGET_HOST}
-          port #{TARGET_PORT}
+          port #{@target_port}
         </server>
       ]
 
@@ -232,7 +238,7 @@ EOL
         tls_cert_logical_store_name Root
         <server>
           host #{TARGET_HOST}
-          port #{TARGET_PORT}
+          port #{@target_port}
         </server>
       ]
 
@@ -250,7 +256,7 @@ EOL
         tls_cert_thumbprint a909502dd82ae41433e6f83886b00d4277a32a7b
         <server>
           host #{TARGET_HOST}
-          port #{TARGET_PORT}
+          port #{@target_port}
         </server>
       ]
 
@@ -260,6 +266,58 @@ EOL
     end
   end
 
+  test 'server is an abbreviation of static type of service_discovery' do
+    @d = d = create_driver(%[
+<server>
+  host 127.0.0.1
+  port 1234
+</server>
+
+<service_discovery>
+  @type static
+
+  <service>
+    host 127.0.0.1
+    port 1235
+  </service>
+</service_discovery>
+    ])
+
+
+    assert_equal(
+      [
+        { host: '127.0.0.1', port: 1234 },
+        { host: '127.0.0.1', port: 1235 },
+      ],
+      d.instance.discovery_manager.services.collect do |service|
+        { host: service.host, port: service.port }
+      end
+    )
+  end
+
+  test 'pass username and password as empty string to HandshakeProtocol' do
+    config_path = File.join(TMP_DIR, "sd_file.conf")
+    File.open(config_path, 'w') do |file|
+      file.write(%[
+- 'host': 127.0.0.1
+  'port': 1234
+  'weight': 1
+])
+    end
+
+    mock(Fluent::Plugin::ForwardOutput::HandshakeProtocol).new(log: anything, hostname: nil, shared_key: anything, password: '', username: '')
+    @d = d = create_driver(%[
+<service_discovery>
+  @type file
+  path #{config_path}
+</service_discovery>
+    ])
+
+    assert_equal 1, d.instance.discovery_manager.services.size
+    assert_equal '127.0.0.1', d.instance.discovery_manager.services[0].host
+    assert_equal 1234, d.instance.discovery_manager.services[0].port
+  end
+
   test 'compress_default_value' do
     @d = d = create_driver
     assert_equal :text, d.instance.compress
@@ -269,7 +327,7 @@ EOL
   end
 
   test 'set_compress_is_gzip' do
-    @d = d = create_driver(CONFIG + %[compress gzip])
+    @d = d = create_driver(config + %[compress gzip])
     assert_equal :gzip, d.instance.compress
     assert_equal :gzip, d.instance.buffer.compress
 
@@ -281,7 +339,7 @@ EOL
     mock = flexmock($log)
     mock.should_receive(:log).with("buffer is compressed.  If you also want to save the bandwidth of a network, Add `compress` configuration in <match>")
 
-    @d = d = create_driver(CONFIG + %[
+    @d = d = create_driver(config + %[
        <buffer>
          type memory
          compress gzip
@@ -295,7 +353,7 @@ EOL
   end
 
   test 'phi_failure_detector disabled' do
-    @d = d = create_driver(CONFIG + %[phi_failure_detector false \n phi_threshold 0])
+    @d = d = create_driver(config + %[phi_failure_detector false \n phi_threshold 0])
     node = d.instance.nodes.first
     stub(node.failure).phi { raise 'Should not be called' }
     node.tick
@@ -303,34 +361,55 @@ EOL
   end
 
   test 'phi_failure_detector enabled' do
-    @d = d = create_driver(CONFIG + %[phi_failure_detector true \n phi_threshold 0])
+    @d = d = create_driver(config + %[phi_failure_detector true \n phi_threshold 0])
     node = d.instance.nodes.first
     node.tick
     assert_false node.available?
   end
 
   test 'require_ack_response is disabled in default' do
-    @d = d = create_driver(CONFIG)
+    @d = d = create_driver(config)
     assert_equal false, d.instance.require_ack_response
     assert_equal 190, d.instance.ack_response_timeout
   end
 
   test 'require_ack_response can be enabled' do
-    @d = d = create_driver(CONFIG + %[
+    @d = d = create_driver(config + %[
       require_ack_response true
       ack_response_timeout 2s
     ])
+    d.instance_start
     assert d.instance.require_ack_response
     assert_equal 2, d.instance.ack_response_timeout
   end
 
+  test 'suspend_flush is disable before before_shutdown' do
+    @d = d = create_driver(config + %[
+      require_ack_response true
+      ack_response_timeout 2s
+    ])
+    d.instance_start
+    assert_false d.instance.instance_variable_get(:@suspend_flush)
+  end
+
+  test 'suspend_flush should be enabled and try_flush returns nil after before_shutdown' do
+    @d = d = create_driver(config + %[
+      require_ack_response true
+      ack_response_timeout 2s
+    ])
+    d.instance_start
+    d.instance.before_shutdown
+    assert_true d.instance.instance_variable_get(:@suspend_flush)
+    assert_nil d.instance.try_flush
+  end
+
   test 'verify_connection_at_startup is disabled in default' do
-    @d = d = create_driver(CONFIG)
+    @d = d = create_driver(config)
     assert_false d.instance.verify_connection_at_startup
   end
 
   test 'verify_connection_at_startup can be enabled' do
-    @d = d = create_driver(CONFIG + %[
+    @d = d = create_driver(config + %[
       verify_connection_at_startup true
     ])
     assert_true d.instance.verify_connection_at_startup
@@ -339,7 +418,7 @@ EOL
   test 'send tags in str (utf-8 strings)' do
     target_input_driver = create_target_input_driver
 
-    @d = d = create_driver(CONFIG + %[flush_interval 1s])
+    @d = d = create_driver(config + %[flush_interval 1s])
 
     time = event_time("2011-01-02 13:14:15 UTC")
 
@@ -372,7 +451,7 @@ EOL
   test 'send_with_time_as_integer' do
     target_input_driver = create_target_input_driver
 
-    @d = d = create_driver(CONFIG + %[flush_interval 1s])
+    @d = d = create_driver(config + %[flush_interval 1s])
 
     time = event_time("2011-01-02 13:14:15 UTC")
 
@@ -400,7 +479,7 @@ EOL
   test 'send_without_time_as_integer' do
     target_input_driver = create_target_input_driver
 
-    @d = d = create_driver(CONFIG + %[
+    @d = d = create_driver(config + %[
       flush_interval 1s
       time_as_integer false
     ])
@@ -430,7 +509,7 @@ EOL
   test 'send_comprssed_message_pack_stream_if_compress_is_gzip' do
     target_input_driver = create_target_input_driver
 
-    @d = d = create_driver(CONFIG + %[
+    @d = d = create_driver(config + %[
       flush_interval 1s
       compress gzip
     ])
@@ -460,7 +539,7 @@ EOL
   test 'send_to_a_node_supporting_responses' do
     target_input_driver = create_target_input_driver
 
-    @d = d = create_driver(CONFIG + %[flush_interval 1s])
+    @d = d = create_driver(config + %[flush_interval 1s])
 
     time = event_time("2011-01-02 13:14:15 UTC")
 
@@ -486,7 +565,7 @@ EOL
   test 'send_to_a_node_not_supporting_responses' do
     target_input_driver = create_target_input_driver
 
-    @d = d = create_driver(CONFIG + %[flush_interval 1s])
+    @d = d = create_driver(config + %[flush_interval 1s])
 
     time = event_time("2011-01-02 13:14:15 UTC")
 
@@ -512,38 +591,95 @@ EOL
   test 'a node supporting responses' do
     target_input_driver = create_target_input_driver
 
-    @d = d = create_driver(CONFIG + %[
+    @d = d = create_driver(config + %[
       require_ack_response true
       ack_response_timeout 1s
       <buffer tag>
         flush_mode immediate
         retry_type periodic
         retry_wait 30s
-        flush_at_shutdown false # suppress errors in d.instance_shutdown
+        flush_at_shutdown true
       </buffer>
     ])
 
     time = event_time("2011-01-02 13:14:15 UTC")
 
     acked_chunk_ids = []
+    nacked = false
     mock.proxy(d.instance.ack_handler).read_ack_from_sock(anything) do |info, success|
       if success
         acked_chunk_ids << info.chunk_id
+      else
+        nacked = true
       end
-      [chunk_id, success]
+      [info, success]
     end
 
     records = [
       {"a" => 1},
       {"a" => 2}
     ]
-    target_input_driver.run(expect_records: 2) do
-      d.end_if { acked_chunk_ids.size > 0 }
+    target_input_driver.run(expect_records: 2, timeout: 5) do
+      d.end_if { acked_chunk_ids.size > 0 || nacked }
       d.run(default_tag: 'test', wait_flush_completion: false, shutdown: false) do
         d.feed([[time, records[0]], [time,records[1]]])
       end
     end
 
+    assert(!nacked, d.instance.log.logs.join)
+
+    events = target_input_driver.events
+    assert_equal ['test', time, records[0]], events[0]
+    assert_equal ['test', time, records[1]], events[1]
+
+    assert_equal 1, acked_chunk_ids.size
+    assert_equal d.instance.sent_chunk_ids.first, acked_chunk_ids.first
+  end
+
+  test 'a node supporting responses after stop' do
+    target_input_driver = create_target_input_driver
+
+    @d = d = create_driver(config + %[
+      require_ack_response true
+      ack_response_timeout 10s
+      <buffer tag>
+        flush_mode immediate
+        retry_type periodic
+        retry_wait 30s
+        flush_at_shutdown true
+      </buffer>
+    ])
+
+    time = event_time("2011-01-02 13:14:15 UTC")
+
+    acked_chunk_ids = []
+    nacked = false
+    mock.proxy(d.instance.ack_handler).read_ack_from_sock(anything) do |info, success|
+      if success
+        acked_chunk_ids << info.chunk_id
+      else
+        nacked = true
+      end
+      [info, success]
+    end
+
+    records = [
+      {"a" => 1},
+      {"a" => 2}
+    ]
+    target_input_driver.run(expect_records: 2, timeout: 5) do
+      d.end_if { acked_chunk_ids.size > 0 || nacked }
+      d.run(default_tag: 'test', wait_flush_completion: false, shutdown: false) do
+        d.instance.stop
+        d.feed([[time, records[0]], [time,records[1]]])
+        d.instance.before_shutdown
+        d.instance.shutdown
+        d.instance.after_shutdown
+      end
+    end
+
+    assert(!nacked, d.instance.log.logs.join)
+
     events = target_input_driver.events
     assert_equal ['test', time, records[0]], events[0]
     assert_equal ['test', time, records[1]], events[1]
@@ -557,7 +693,7 @@ EOL
   test 'TLS transport and ack parameter combination' do |ack|
     omit "TLS and 'ack false' always fails on AppVeyor. Need to debug" if Fluent.windows? && !ack
 
-    input_conf = TARGET_CONFIG + %[
+    input_conf = target_config + %[
                    <transport tls>
                      insecure true
                    </transport>
@@ -571,7 +707,7 @@ EOL
       tls_insecure_mode true
       <server>
         host #{TARGET_HOST}
-        port #{TARGET_PORT}
+        port #{@target_port}
       </server>
       <buffer>
         #flush_mode immediate
@@ -600,7 +736,7 @@ EOL
   test 'a destination node not supporting responses by just ignoring' do
     target_input_driver = create_target_input_driver(response_stub: ->(_option) { nil }, disconnect: false)
 
-    @d = d = create_driver(CONFIG + %[
+    @d = d = create_driver(config + %[
       require_ack_response true
       ack_response_timeout 1s
       <buffer tag>
@@ -645,9 +781,9 @@ EOL
   test 'a destination node not supporting responses by disconnection' do
     target_input_driver = create_target_input_driver(response_stub: ->(_option) { nil }, disconnect: true)
 
-    @d = d = create_driver(CONFIG + %[
+    @d = d = create_driver(config + %[
       require_ack_response true
-      ack_response_timeout 5s
+      ack_response_timeout 1s
       <buffer tag>
         flush_mode immediate
         retry_type periodic
@@ -675,7 +811,7 @@ EOL
       end
     end
 
-    assert_equal (5 + 2), delayed_commit_timeout_value
+    assert_equal (1 + 2), delayed_commit_timeout_value
 
     events = target_input_driver.events
     assert_equal ['test', time, records[0]], events[0]
@@ -688,7 +824,7 @@ EOL
   end
 
   test 'authentication_with_shared_key' do
-    input_conf = TARGET_CONFIG + %[
+    input_conf = target_config + %[
                    <security>
                      self_hostname in.localhost
                      shared_key fluentd-sharedkey
@@ -708,7 +844,7 @@ EOL
       <server>
         name test
         host #{TARGET_HOST}
-        port #{TARGET_PORT}
+        port #{@target_port}
         shared_key fluentd-sharedkey
       </server>
     ]
@@ -735,7 +871,7 @@ EOL
   end
 
   test 'keepalive + shared_key' do
-    input_conf = TARGET_CONFIG + %[
+    input_conf = target_config + %[
                    <security>
                      self_hostname in.localhost
                      shared_key fluentd-sharedkey
@@ -753,7 +889,7 @@ EOL
       <server>
         name test
         host #{TARGET_HOST}
-        port #{TARGET_PORT}
+        port #{@target_port}
       </server>
     ]
     @d = d = create_driver(output_conf)
@@ -783,7 +919,7 @@ EOL
   end
 
    test 'authentication_with_user_auth' do
-    input_conf = TARGET_CONFIG + %[
+    input_conf = target_config + %[
                    <security>
                      self_hostname in.localhost
                      shared_key fluentd-sharedkey
@@ -808,7 +944,7 @@ EOL
       <server>
         name test
         host #{TARGET_HOST}
-        port #{TARGET_PORT}
+        port #{@target_port}
         shared_key fluentd-sharedkey
         username fluentd
         password fluentd
@@ -838,7 +974,7 @@ EOL
 
   # This test is not 100% but test failed with previous Node implementation which has race condition
   test 'Node with security is thread-safe on multi threads' do
-    input_conf = TARGET_CONFIG + %[
+    input_conf = target_config + %[
                    <security>
                      self_hostname in.localhost
                      shared_key fluentd-sharedkey
@@ -857,7 +993,7 @@ EOL
       <server>
         name test
         host #{TARGET_HOST}
-        port #{TARGET_PORT}
+        port #{@target_port}
         shared_key fluentd-sharedkey
       </server>
     ]
@@ -878,10 +1014,12 @@ EOL
     end
 
     logs = d.logs
-    assert_false(logs.any? { |log| log.include?("invalid format for PONG message") || log.include?("shared key mismatch") }, "'#{logs.last.strip}' happens")
+    assert_false(logs.any? { |log|
+                   log.include?("invalid format for PONG message") || log.include?("shared key mismatch")
+                 }, "Actual log:\n#{logs.join}")
   end
 
-  def create_target_input_driver(response_stub: nil, disconnect: false, conf: TARGET_CONFIG)
+  def create_target_input_driver(response_stub: nil, disconnect: false, conf: target_config)
     require 'fluent/plugin/in_forward'
 
     # TODO: Support actual TCP heartbeat test
@@ -897,7 +1035,7 @@ EOL
   end
 
   test 'heartbeat_type_none' do
-    @d = d = create_driver(CONFIG + "\nheartbeat_type none")
+    @d = d = create_driver(config + "\nheartbeat_type none")
     node = d.instance.nodes.first
     assert_equal Fluent::Plugin::ForwardOutput::NoneHeartbeatNode, node.class
 
@@ -911,7 +1049,7 @@ EOL
   end
 
   test 'heartbeat_type_udp' do
-    @d = d = create_driver(CONFIG + "\nheartbeat_type udp")
+    @d = d = create_driver(config + "\nheartbeat_type udp")
 
     d.instance_start
     usock = d.instance.instance_variable_get(:@usock)
@@ -922,7 +1060,7 @@ EOL
     assert servers.find{|s| s.title == :out_forward_heartbeat_receiver }
     assert timers.include?(:out_forward_heartbeat_request)
 
-    mock(usock).send("\0", 0, Socket.pack_sockaddr_in(TARGET_PORT, '127.0.0.1')).once
+    mock(usock).send("\0", 0, Socket.pack_sockaddr_in(@target_port, '127.0.0.1')).once
     d.instance.send(:on_heartbeat_timer)
   end
 
@@ -945,21 +1083,45 @@ EOL
     end
   end
 
+  test 'when out_forward has @id' do
+    # cancel https://github.com/fluent/fluentd/blob/077508ac817b7637307434d0c978d7cdc3d1c534/lib/fluent/plugin_id.rb#L43-L53
+    # it always return true in test
+    mock.proxy(Fluent::Plugin).new_sd('static', parent: anything) { |v|
+      stub(v).plugin_id_for_test? { false }
+    }.once
+
+    output = Fluent::Test::Driver::Output.new(Fluent::Plugin::ForwardOutput) {
+      def plugin_id_for_test?
+        false
+      end
+    }
+
+    assert_nothing_raised do
+      output.configure(config + %[
+        @id unique_out_forward
+      ])
+    end
+  end
+
   sub_test_case 'verify_connection_at_startup' do
     test 'nodes are not available' do
-      @d = d = create_driver(CONFIG + %[
+      @d = d = create_driver(config + %[
         verify_connection_at_startup true
       ])
       e = assert_raise Fluent::UnrecoverableError do
         d.instance_start
       end
-      assert_match(/Connection refused/, e.message)
+      if Fluent.windows?
+        assert_match(/No connection could be made because the target machine actively refused it/, e.message)
+      else
+        assert_match(/Connection refused/, e.message)
+      end
 
       d.instance_shutdown
     end
 
     test 'nodes_shared_key_miss_match' do
-      input_conf = TARGET_CONFIG + %[
+      input_conf = target_config + %[
                    <security>
                      self_hostname in.localhost
                      shared_key fluentd-sharedkey
@@ -976,7 +1138,7 @@ EOL
 
         <server>
           host #{TARGET_HOST}
-          port #{TARGET_PORT}
+          port #{@target_port}
         </server>
       ]
       @d = d = create_driver(output_conf)
@@ -985,12 +1147,12 @@ EOL
         e = assert_raise Fluent::UnrecoverableError do
           d.instance_start
         end
-        assert_match(/Failed to establish connection/, e.message)
+        assert_match(/failed to establish connection/, e.message)
       end
     end
 
     test 'nodes_shared_key_miss_match with TLS' do
-      input_conf = TARGET_CONFIG + %[
+      input_conf = target_config + %[
                    <security>
                      self_hostname in.localhost
                      shared_key fluentd-sharedkey
@@ -1011,7 +1173,7 @@ EOL
 
         <server>
           host #{TARGET_HOST}
-          port #{TARGET_PORT}
+          port #{@target_port}
         </server>
       ]
       @d = d = create_driver(output_conf)
@@ -1021,12 +1183,12 @@ EOL
           d.instance_start
         end
 
-        assert_match(/Failed to establish connection/, e.message)
+        assert_match(/failed to establish connection/, e.message)
       end
     end
 
     test 'nodes_shared_key_match' do
-      input_conf = TARGET_CONFIG + %[
+      input_conf = target_config + %[
                        <security>
                          self_hostname in.localhost
                          shared_key fluentd-sharedkey
@@ -1042,7 +1204,7 @@ EOL
           <server>
             name test
             host #{TARGET_HOST}
-            port #{TARGET_PORT}
+            port #{@target_port}
           </server>
       ]
       @d = d = create_driver(output_conf)
@@ -1066,14 +1228,19 @@ EOL
   end
 
   test 'Create new connection per send_data' do
-    target_input_driver = create_target_input_driver(conf: TARGET_CONFIG)
-    output_conf = CONFIG
+    target_input_driver = create_target_input_driver(conf: target_config)
+    output_conf = config
     d = create_driver(output_conf)
     d.instance_start
 
     begin
       chunk = Fluent::Plugin::Buffer::MemoryChunk.new(Fluent::Plugin::Buffer::Metadata.new(nil, nil, nil))
-      mock.proxy(d.instance).socket_create_tcp(TARGET_HOST, TARGET_PORT, anything) { |sock| mock(sock).close.once; sock }.twice
+      mock.proxy(d.instance).socket_create_tcp(TARGET_HOST, @target_port,
+                                               linger_timeout: anything,
+                                               send_timeout: anything,
+                                               recv_timeout: anything,
+                                               connect_timeout: anything
+                                              ) { |sock| mock(sock).close.once; sock }.twice
 
       target_input_driver.run(timeout: 15) do
         d.run(shutdown: false) do
@@ -1095,7 +1262,7 @@ EOL
       name test
       standby
       host #{TARGET_HOST}
-      port #{TARGET_PORT}
+      port #{@target_port}
     </server>
     ])
     d.instance_start
@@ -1104,8 +1271,8 @@ EOL
 
   sub_test_case 'keepalive' do
     test 'Do not create connection per send_data' do
-      target_input_driver = create_target_input_driver(conf: TARGET_CONFIG)
-      output_conf = CONFIG + %[
+      target_input_driver = create_target_input_driver(conf: target_config)
+      output_conf = config + %[
         keepalive true
         keepalive_timeout 2
       ]
@@ -1114,7 +1281,12 @@ EOL
 
       begin
         chunk = Fluent::Plugin::Buffer::MemoryChunk.new(Fluent::Plugin::Buffer::Metadata.new(nil, nil, nil))
-        mock.proxy(d.instance).socket_create_tcp(TARGET_HOST, TARGET_PORT, anything) { |sock| mock(sock).close.once; sock }.once
+        mock.proxy(d.instance).socket_create_tcp(TARGET_HOST, @target_port,
+                                                 linger_timeout: anything,
+                                                 send_timeout: anything,
+                                                 recv_timeout: anything,
+                                                 connect_timeout: anything
+                                                ) { |sock| mock(sock).close.once; sock }.once
 
         target_input_driver.run(timeout: 15) do
           d.run(shutdown: false) do
@@ -1129,10 +1301,19 @@ EOL
       end
     end
 
+    test 'create timer of purging obsolete sockets' do
+      output_conf = config + %[keepalive true]
+      d = create_driver(output_conf)
+
+      mock(d.instance).timer_execute(:out_forward_heartbeat_request, 1).once
+      mock(d.instance).timer_execute(:out_forward_keep_alived_socket_watcher, 5).once
+      d.instance_start
+    end
+
     sub_test_case 'with require_ack_response' do
       test 'Create connection per send_data' do
-        target_input_driver = create_target_input_driver(conf: TARGET_CONFIG)
-        output_conf = CONFIG + %[
+        target_input_driver = create_target_input_driver(conf: target_config)
+        output_conf = config + %[
           require_ack_response true
           keepalive true
           keepalive_timeout 2
@@ -1142,7 +1323,13 @@ EOL
 
         begin
           chunk = Fluent::Plugin::Buffer::MemoryChunk.new(Fluent::Plugin::Buffer::Metadata.new(nil, nil, nil))
-          mock.proxy(d.instance).socket_create_tcp(TARGET_HOST, TARGET_PORT, anything) { |sock| mock(sock).close.once; sock }.twice
+          mock.proxy(d.instance).socket_create_tcp(TARGET_HOST, @target_port,
+                                                   linger_timeout: anything,
+                                                   send_timeout: anything,
+                                                   recv_timeout: anything,
+                                                   connect_timeout: anything) { |sock|
+            mock(sock).close.once; sock
+          }.twice
 
           target_input_driver.run(timeout: 15) do
             d.run(shutdown: false) do
